; #########################################################################

DecToQW    proc syscall

  ; ----------------------------------------------------------
  ; Convert positive decimal string into QWORD value by IanB
  ; using floating point unit

  ; On entry:
  ;     string pointer in ECX, string must be zero-terminated
  ; On exit:
  ;     converted value in EAX (low DWORD)/ EDX (high DWORD)
  ;     ECX is zero if there is an error in string or overflow,
  ;     else non-zero to show successful conversion
  ; EBX/EBP/EDI/ESI preserved
  ; ----------------------------------------------------------

        finit
        xor     eax, eax
        push    eax                     ; load 32 bits of zeros on stack as FP variable

        mov     al, BYTE PTR [ecx]
        add     ecx, 1
        mov     edx, 19                 ; max decimal digits for (2^63 - 1) which is upper integer limit

        sub     al, 30H
        jb      BadNumericString        ; not numeric digit
        cmp     al, 9
        ja      BadNumericString        ; not numeric digit

        push    eax                     ; load second 32 bits on stack as FP variable
        fild    QWORD PTR [esp]         ; set starting value for FPU
        mov     WORD PTR [esp+2], 10    ; set multiplier value

NewASCIIDigitLoop:
        mov     al, BYTE PTR [ecx]
        add     ecx, 1

        test    al, al
        jz      EndASCIIDigitLoop       ; found string end
                                        ; other tests could be inserted here, eg. if number string is not
                                        ; null-terminated but in a line of text, or for thousand-delimiters

        fimul   WORD PTR [esp+2]        ; mul by 10

        sub     al, 30H
        jb      BadNumericString        ; not numeric digit
        cmp     al, 9
        ja      BadNumericString        ; not numeric digit

        mov     BYTE PTR [esp], al
        fiadd   WORD PTR [esp]          ; add new digit when mul has finished

        sub     edx, 1
        jnz     NewASCIIDigitLoop
        jmp     BadNumericString        ; too many digits

EndASCIIDigitLoop:
        or      [esp], 0ffffffffH       ; max integer value is (2^63 - 1)
        mov     [esp+4], 7fffffffH
        fild    QWORD PTR [esp]         ; load comparison value in FPU
        fcomp   st(1)                   ; compare then pop it

        fstsw   ax                      ; copy the status register containing the result to AX
        fwait
        sahf                            ; transfer condition codes to the CPU's flag register
        jpe     BadNumericString
        jb      BadNumericString

        fistp   QWORD PTR [esp]         ; retrieve the final value
        or      ecx, 0ffffffffH         ; show successful result by non-zero in ECX
        fwait
        pop     eax
        pop     edx

        ret

BadNumericString:
        ffree   st(0)
        pop     eax                     ; retrieve values off stack, will not be meaningful
        pop     edx
        xor     ecx, ecx                ; show non-successful result by zero in ECX
        ret

DecToQW    endp

; #########################################################################
